local TheSim = GLOBAL.TheSim
local Vector3 = GLOBAL.Vector3
local SpawnPrefab = GLOBAL.SpawnPrefab

local function CheckDlcEnabled(dlc)
    -- if the constant doesn't even exist, then they can't have the DLC
    if not GLOBAL.rawget(GLOBAL, dlc) then return false end
    GLOBAL.assert(GLOBAL.rawget(GLOBAL, "IsDLCEnabled"), "Old version of game, please update (IsDLCEnabled function missing)")
    return GLOBAL.IsDLCEnabled(GLOBAL[dlc])
end

local DST = GLOBAL.TheSim.GetGameID ~= nil and GLOBAL.TheSim:GetGameID() == "DST"
local ROG = DST or CheckDlcEnabled("REIGN_OF_GIANTS")
local CSW = CheckDlcEnabled("CAPY_DLC")
local HML = CheckDlcEnabled("PORKLAND_DLC")

-- 物品堆疊最大化
local STACK_SIZE_MAX = 64;
TUNING.STACK_SIZE_LARGEITEM = STACK_SIZE_MAX
TUNING.STACK_SIZE_MEDITEM = STACK_SIZE_MAX
TUNING.STACK_SIZE_SMALLITEM = STACK_SIZE_MAX

TUNING.TRAP_TEETH_DAMAGE = 500

TUNING.WILSON_HEALTH = TUNING.WILSON_HEALTH * 2
TUNING.WILSON_HUNGER = TUNING.WILSON_HUNGER * 2
TUNING.WILSON_SANITY = TUNING.WILSON_SANITY * 1

TUNING.WX78_MAX_HEALTH = TUNING.WX78_MAX_HEALTH * 5
TUNING.WX78_MAX_HUNGER = TUNING.WX78_MAX_HUNGER * 5
TUNING.WX78_MAX_SANITY = TUNING.WX78_MAX_SANITY * 5

TUNING.FISHING_MINWAIT = 1
TUNING.FISHING_MAXWAIT = 1

-- 讓冰箱的東西不會腐壞
TUNING.PERISH_FRIDGE_MULT = 0

-- 快速撿取
local pickPrefabs = {
    "sapling",
    "marsh_bush",
    "reeds",
    "grass",
    "berrybush",
    "berrybush2",
    "flower_cave",
    "red_mushroom",
    "green_mushroom",
    "blue_mushroom",
    "cactus",
    "lichen",
    "seashell",
    "seashell_beached",
    "seaweed",
    "seaweed_planted",
    "limpets",
    "tumbleweed",
    "beebox"
}

function QuickPick(inst)
    if inst.components.pickable then
        inst.components.pickable.quickpick = true
    end
end
for i, prefab in ipairs(pickPrefabs) do
    AddPrefabPostInit(prefab, QuickPick)
end

-- 提升傷害
function DamageUp(inst)
    if inst.components.weapon then
        inst.components.weapon:SetDamage(450)
    end
end

-- 提升移動速度
function SpeedUp(inst)
    if inst.components.equippable then
        inst.components.equippable.walkspeedmult = 1.4
    end
end

-- 無限使用
function InfiniteUse(inst)
    if inst.components.finiteuses then
        inst:RemoveComponent("finiteuses")
    end
end

local tools = {
    "axe",
    "shovel",
    "pickaxe",
    "hammer",
    "pitchfork",
    "bugnet",
    "fishingrod"
}
for i, tool in ipairs(tools) do
    AddPrefabPostInit(tool, DamageUp)
    AddPrefabPostInit(tool, SpeedUp)
    AddPrefabPostInit(tool, InfiniteUse)
end

-- 超級斧頭
function superAxe(inst)
    if inst.components.tool then
        inst.components.tool:SetAction(GLOBAL.ACTIONS.CHOP, 30)
        inst.components.tool:SetAction(GLOBAL.ACTIONS.MINE, 30)
    end
end
AddPrefabPostInit("axe", superAxe)

-- 超級草帽
local function hat_equip(inst, owner, fname_override)
    local build = fname_override or "hat_straw"
    owner.AnimState:OverrideSymbol("swap_hat", build, "swap_hat")
    owner.AnimState:Show("HAT")
    owner.AnimState:Show("HAT_HAIR")
    inst.Light:Enable(true)
end

local function hat_unequip(inst, owner)
    owner.AnimState:Hide("HAT")
    owner.AnimState:Hide("HAT_HAIR")
    inst.Light:Enable(false)
end

local function SuperHat(inst)
    
    -- 移除使用腐爛
    if inst.components.perishable then
        inst:RemoveComponent("perishable")
    end
    
    -- 移除使用限制
    if inst.components.finiteuses then
        inst:RemoveComponent("finiteuses")
    end
    
    -- 無限使用
    if inst.components.fueled then
        inst:RemoveComponent("fueled")
    end
    
    -- 只有在有資料片才啟用新增此功能
    if ROG or CSW then
        -- 新增防水功能閃電絕緣
        inst:AddComponent("waterproofer")
        inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_ABSOLUTE)
        -- 新增保暖功能
        inst:AddComponent("insulator")
        inst.components.insulator:SetInsulation(TUNING.INSULATION_LARGE)
    end
    -- 新增發光功能
    local light = inst.entity:AddLight()
    
    light:SetFalloff(0.33)
    light:SetIntensity(.8)
    light:SetRadius(5)
    light:SetColour(255 / 255, 255 / 255, 255 / 255)
    
    light:Enable(false)
    
    if inst.components.equippable then
        inst.components.equippable.equipslot = "head"
        -- inst.components.equippable:SetOnEquip(hat_equip)
        -- inst.components.equippable:SetOnUnequip(hat_unequip)
        inst.components.equippable.insulated = true
        inst.components.equippable.walkspeedmult = 2.5
    end

end

AddPrefabPostInit("strawhat", SuperHat)

-- 自動接回力鏢
function setAutoCatch(inst)
    if inst.components.projectile then
        inst.components.projectile:SetSpeed(50);
        local oldhit = inst.components.projectile.Hit
        function inst.components.projectile:Hit(target)
            if target == self.owner and target.components.catcher then
                target:PushEvent("catch", {projectile = self.inst})
                self.inst:PushEvent("caught", {catcher = target})
                self:Catch(target)
                target.components.catcher:StopWatching(self.inst)
            else
                oldhit(self, target)
            end
        end
    end
end

AddPrefabPostInit("boomerang", InfiniteUse)
AddPrefabPostInit("boomerang", DamageUp)
AddPrefabPostInit("boomerang", SpeedUp)
AddPrefabPostInit("boomerang", setAutoCatch)

AddPrefabPostInit("bonerang", InfiniteUse)
AddPrefabPostInit("bonerang", DamageUp)
AddPrefabPostInit("bonerang", SpeedUp)
AddPrefabPostInit("bonerang", setAutoCatch)

-- 自動重新放置陷阱
local function TrapComponentPostInit(self)
    function self:Harvest(doer)
        if self.issprung then
            self.inst:PushEvent("harvesttrap")
            if self.onharvest then
                self.onharvest(self.inst)
            end
            
            local timeintrap = self.inst.components.timer and self.inst.components.timer:GetTimeElapsed("foodspoil")
            if self.lootprefabs and doer.components.inventory then
                for k, v in ipairs(self.lootprefabs) do
                    local loot = SpawnPrefab(v)
                    if loot then
                        doer.components.inventory:GiveItem(loot, nil, Vector3(TheSim:GetScreenPos(self.inst.Transform:GetWorldPosition())))
                        if loot.components.perishable and timeintrap then
                            loot.components.perishable:LongUpdate(timeintrap)
                        end
                    end
                end
            end
            self:Reset()
            self.inst:PushEvent("ondropped")
        else
            if self.inst.components.finiteuses and self.inst.components.finiteuses:GetUses() > 0 then
                doer.components.inventory:GiveItem(self.inst, nil, Vector3(TheSim:GetScreenPos(self.inst.Transform:GetWorldPosition())))
            end
        end
    end
end
AddPrefabPostInit("trap", InfiniteUse)
AddComponentPostInit("trap", TrapComponentPostInit)
